@@ -22,6 +22,8 @@
 
 #include "applet-struct.h"
 #include "applet-notifications.h"
+#include "applet-run-dialog.h"
+#include "applet-menu.h"
 #include "applet-config.h"
 
 
@@ -47,36 +49,7 @@ CD_APPLET_RESET_CONFIG_END
 
 //\_________________ Here you have to free all ressources allocated for myData. This one will be reseted to 0 at the end of this function. This function is called when your applet is stopped, in the very end.
 CD_APPLET_RESET_DATA_BEGIN
-	if (myData.pMenu)
-		gtk_widget_destroy (myData.pMenu);  // detruit aussi pRecentMenuItem.
+	cd_menu_stop ();
 	
-	if (myData.loaded_icons != NULL)
-	{
-		g_hash_table_destroy (myData.loaded_icons);  // wll unref objects inside.
-		myData.loaded_icons = NULL;  // set to NULL for the callback
-	}
-	
-	if (myData.image_menu_items != NULL)
-	{
-		g_slist_free (myData.image_menu_items);  // objects inside will be destroyed with the menu.
-		myData.image_menu_items = NULL;
-	}
-	
-	if (myData.dir_hash)
-		g_hash_table_destroy (myData.dir_hash);
-	
-	GList *l;
-	for (l = myData.possible_executables; l; l = l->next)
-		g_free (l->data);
-	g_list_free (myData.possible_executables);
-	
-	for (l = myData.completion_items; l; l = l->next)
-		g_free (l->data);
-	g_list_free (myData.completion_items);
-	
-	if (myData.completion)
-		g_completion_free (myData.completion);
-	
-	if (!cairo_dock_dialog_unreference (myData.pQuickLaunchDialog))
-		cairo_dock_dialog_unreference (myData.pQuickLaunchDialog);
+	cd_run_dialog_free ();
 CD_APPLET_RESET_DATA_END
