@@ -29,8 +29,10 @@ CD_APPLET_GET_CONFIG_BEGIN
 	myConfig.defaultTitle = CD_CONFIG_GET_STRING ("Icon", "name");
 	myConfig.cAnimationName = CD_CONFIG_GET_STRING ("Configuration", "animation");
 	myConfig.cShortkey = CD_CONFIG_GET_STRING ("Configuration", "shortkey");
-	#ifdef INDICATOR_MESSAGES_12_10
-	myConfig.cIndicatorName = CD_CONFIG_GET_STRING_WITH_DEFAULT ("Configuration", "indicator", "libmessaging.so");
+	#ifdef INDICATOR_MESSAGES_WITH_IND3
+	myConfig.cIndicatorName = CD_CONFIG_GET_STRING ("Configuration", "indicator name");
+	if (myConfig.cIndicatorName == NULL)
+		myConfig.cIndicatorName = g_strdup ("libmessaging.so");
 	#endif
 CD_APPLET_GET_CONFIG_END
 
@@ -40,7 +42,7 @@ CD_APPLET_RESET_CONFIG_BEGIN
 	g_free (myConfig.cShortkey);
 	g_free (myConfig.cAnimationName);
 	g_free (myConfig.defaultTitle);
-	#ifdef INDICATOR_MESSAGES_12_10
+	#ifdef INDICATOR_MESSAGES_WITH_IND3
 	g_free (myConfig.cIndicatorName);
 	#endif
 CD_APPLET_RESET_CONFIG_END
