@@ -23,7 +23,7 @@
 
 #include <cairo-dock.h>
 
-#ifndef INDICATOR_MESSAGES_12_10
+#ifndef INDICATOR_MESSAGES_WITH_IND3
 #include "indicator-applet.h"
 #define FORCE_REMOVE_DOUBLE_SEPARATORS
 // let's include the Dbus name shere, so that we don't duplicate the logic.
@@ -44,14 +44,14 @@ struct _AppletConfig {
 	gchar *cAnimationName;
 	gchar *cShortkey;
 	gchar *defaultTitle;
-	#ifdef INDICATOR_MESSAGES_12_10
+	#ifdef INDICATOR_MESSAGES_WITH_IND3
 	gchar *cIndicatorName;
 	#endif
 	} ;
 
 //\___________ structure containing the applet's data, like surfaces, dialogs, results of calculus, etc.
 struct _AppletData {
-	#ifndef INDICATOR_MESSAGES_12_10
+	#ifndef INDICATOR_MESSAGES_WITH_IND3
 	CDAppletIndicator *pIndicator;
 	#else
 	IndicatorObject *pIndicator;
