@@ -87,24 +87,66 @@ static void _cd_logout_check_capabilities_async (CDSharedMemory *pSharedMemory)
 		G_TYPE_INVALID,
 		G_TYPE_BOOLEAN, &pSharedMemory->bCanRestart,
 		G_TYPE_INVALID);
-	if (error)
+	if (!error)
 	{
-		cd_warning ("ConsoleKit error: %s", error->message);
-		g_error_free (error);
-		g_object_unref (pProxy);
-		return;
+		pSharedMemory->iLoginManager = CD_CONSOLE_KIT;
+		
+		dbus_g_proxy_call (pProxy, "CanStop", &error,
+			G_TYPE_INVALID,
+			G_TYPE_BOOLEAN, &pSharedMemory->bCanStop,
+			G_TYPE_INVALID);
+		if (error)
+		{
+			cd_warning ("ConsoleKit error: %s", error->message);
+			g_error_free (error);
+			g_object_unref (pProxy);
+			return;
+		}
 	}
-	
-	dbus_g_proxy_call (pProxy, "CanStop", &error,
-		G_TYPE_INVALID,
-		G_TYPE_BOOLEAN, &pSharedMemory->bCanStop,
-		G_TYPE_INVALID);
-	if (error)
+	else  // console kit is probably not used on this system, try with Logind from Systemd
 	{
-		cd_warning ("ConsoleKit error: %s", error->message);
+		cd_debug ("ConsoleKit error: %s", error->message);
 		g_error_free (error);
-		g_object_unref (pProxy);
-		return;
+		error = NULL;
+		
+		pProxy = cairo_dock_create_new_system_proxy (
+			"org.freedesktop.login1",
+			"/org/freedesktop/login1",
+			"org.freedesktop.login1.Manager");
+		
+		gchar *res;
+		dbus_g_proxy_call (pProxy, "CanReboot", &error,
+			G_TYPE_INVALID,
+			G_TYPE_STRING, &res,
+			G_TYPE_INVALID);
+		if (!error)
+		{
+			pSharedMemory->iLoginManager = CD_SYSTEMD;
+			
+			pSharedMemory->bCanRestart = (res && strcmp (res, "yes") == 0);
+			g_free (res);
+			
+			dbus_g_proxy_call (pProxy, "CanPowerOff", &error,
+				G_TYPE_INVALID,
+				G_TYPE_STRING, &res,
+				G_TYPE_INVALID);
+			if (error)
+			{
+				cd_warning ("Logind error: %s", error->message);
+				g_error_free (error);
+				g_object_unref (pProxy);
+				return;
+			}
+			pSharedMemory->bCanStop = (res && strcmp (res, "yes") == 0);
+			g_free (res);
+		}
+		else
+		{
+			cd_debug ("Logind error: %s", error->message);
+			g_error_free (error);
+			error = NULL;
+		}
+		
 	}
 	g_object_unref (pProxy);
 	
@@ -119,6 +161,10 @@ static void _cd_logout_check_capabilities_async (CDSharedMemory *pSharedMemory)
 		pSharedMemory->bHasGuestAccount = cairo_dock_dbus_get_property_as_boolean (pProxy, "org.freedesktop.DisplayManager.Seat", "HasGuestAccount");
 		g_object_unref (pProxy);
 	}
+	else
+	{
+		pSharedMemory->bHasGuestAccount = cairo_dock_dbus_detect_system_application ("org.gnome.DisplayManager");
+	}
 }
 
 static gboolean _cd_logout_got_capabilities (CDSharedMemory *pSharedMemory)
@@ -132,6 +178,7 @@ static gboolean _cd_logout_got_capabilities (CDSharedMemory *pSharedMemory)
 	myData.bCanRestart = pSharedMemory->bCanRestart;
 	myData.bCanStop = pSharedMemory->bCanStop;
 	myData.bHasGuestAccount = pSharedMemory->bHasGuestAccount;
+	myData.iLoginManager = pSharedMemory->iLoginManager;
 	cd_debug ("capabilities: %d; %d; %d; %d; %d", myData.bCanHibernate, myData.bCanSuspend, myData.bCanRestart, myData.bCanStop, myData.bHasGuestAccount);
 	
 	// display the menu that has been asked beforehand.
@@ -277,7 +324,7 @@ static GtkWidget *_build_menu (void)
 	}
 
 	if ((myDock && myDock->container.bIsHorizontal && ! myDock->container.bDirectionUp)  // on the top, we inverse the menu (mainly to be close to what others do).
-		|| (myDesklet && myDesklet->container.iWindowPositionY < (g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] / 2)))
+		|| (myDesklet && myDesklet->container.iWindowPositionY < (g_desktopGeometry.Xscreen.height / 2)))
 	{
 		GList *children = gtk_container_get_children (GTK_CONTAINER (pMenu));
 		GList *pMenuList;
@@ -300,7 +347,7 @@ static void _display_menu (void)
 	
 	// select the first (or last) item, which corresponds to the 'shutdown' action.
 	if ((myDock && myDock->container.bIsHorizontal && ! myDock->container.bDirectionUp)  // on the top, we inverse the menu
-		|| (myDesklet && myDesklet->container.iWindowPositionY < (g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] / 2)))
+		|| (myDesklet && myDesklet->container.iWindowPositionY < (g_desktopGeometry.Xscreen.height / 2)))
 	{
 		GList *children = gtk_container_get_children (GTK_CONTAINER (pMenu));
 		GList *last_child = g_list_last (children);
@@ -454,7 +501,7 @@ static void _notify_action_required (void)
 
 	g_free (cName);
 
-	gint iIconSize = MAX (myIcon->iImageWidth, myIcon->iImageHeight);
+	gint iIconSize = MAX (myIcon->image.iWidth, myIcon->image.iHeight);
 	gchar *cImagePath = _check_icon (myConfig.cEmblemPath,
 		(myConfig.iRebootNeededImage == CD_DISPLAY_EMBLEM ?
 				iIconSize / 2 :
@@ -599,6 +646,26 @@ static void _console_kit_action (const gchar *cAction)
 	g_object_unref (pProxy);
 }
 
+static void _systemd_action (const gchar *cAction)
+{
+	GError *error = NULL;
+	DBusGProxy *pProxy = cairo_dock_create_new_system_proxy (
+		"org.freedesktop.login1",
+		"/org/freedesktop/login1",
+		"org.freedesktop.login1.Manager");
+	
+	dbus_g_proxy_call (pProxy, cAction, &error,
+		G_TYPE_BOOLEAN, FALSE,  // non-interactive
+		G_TYPE_INVALID,
+		G_TYPE_INVALID);
+	if (error)
+	{
+		cd_warning ("Logind error: %s", error->message);
+		g_error_free (error);
+	}
+	g_object_unref (pProxy);
+}
+
 static void _upower_action (gboolean bSuspend)
 {
 	#ifdef CD_UPOWER_AVAILABLE
@@ -634,7 +701,17 @@ static void _shut_down (void)
 {
 	if (myData.bCanStop)
 	{
-		_console_kit_action ("Stop");  // could use org.gnome.SessionManager.RequestShutdown, but it's not standard.
+		switch (myData.iLoginManager)
+		{
+			case CD_CONSOLE_KIT:
+				_console_kit_action ("Stop");  // could use org.gnome.SessionManager.RequestShutdown, but it's not standard.
+			break;
+			case CD_SYSTEMD:
+				_systemd_action ("PowerOff");
+			break;
+			default:
+			break;
+		}
 	}
 	else if (myConfig.cUserAction2)
 	{
@@ -693,7 +770,17 @@ static void _restart (void)
 {
 	if (myData.bCanRestart)
 	{
-		_console_kit_action ("Restart");  // could use org.gnome.SessionManager.RequestReboot
+		switch (myData.iLoginManager)
+		{
+			case CD_CONSOLE_KIT:
+				_console_kit_action ("Restart");  // could use org.gnome.SessionManager.RequestShutdown, but it's not standard.
+			break;
+			case CD_SYSTEMD:
+				_systemd_action ("Reboot");
+				break;
+			default:
+				break;
+		}
 	}
 	else if (myConfig.cUserAction2)
 	{
@@ -772,6 +859,17 @@ static void cd_logout_switch_to_user (const gchar *cUser)
 		}
 		g_object_unref (pProxy);
 	}
+	else // try with gdm
+	{
+		DBusGProxy *pProxy = cairo_dock_create_new_system_proxy (
+			"org.gnome.DisplayManager",
+			"/org/gnome/DisplayManager/LocalDisplayFactory",
+			"org.gnome.DisplayManager.LocalDisplayFactory");
+		dbus_g_proxy_call_no_reply (pProxy, "SwitchToUser",
+			G_TYPE_STRING, cUser,
+			G_TYPE_INVALID);  // we don't care the 'id' object path returned
+		g_object_unref (pProxy);
+	}
 }
 
 static void cd_logout_switch_to_guest (void)
@@ -795,6 +893,17 @@ static void cd_logout_switch_to_guest (void)
 		}
 		g_object_unref (pProxy);
 	}
+	else // try with gdm
+	{
+		DBusGProxy *pProxy = cairo_dock_create_new_system_proxy (
+			"org.gnome.DisplayManager",
+			"/org/gnome/DisplayManager/LocalDisplayFactory",
+			"org.gnome.DisplayManager.LocalDisplayFactory");
+		dbus_g_proxy_call_no_reply (pProxy, "StartGuestSession",
+			G_TYPE_STRING, "",  // current user session, but actually it can be NULL so why bother?
+			G_TYPE_INVALID);  // we don't care the 'id' object path returned
+		g_object_unref (pProxy);
+	}
 }
 /*
 static gboolean cd_logout_switch_to_greeter (void)  // not really sure how to use it.
@@ -839,6 +948,7 @@ static int _compare_user_name (CDUser *pUser1, CDUser *pUser2)
 {
 	return strcmp (pUser1->cUserName, pUser2->cUserName);
 }
+
 static GList* _get_users_list_fallback (void)
 {
 	// read from /etc/passwd
@@ -861,9 +971,8 @@ static GList* _get_users_list_fallback (void)
 	{
 		cUserProps = g_strsplit (cUsers[i], ":", 0);
 		// add the user if it fits
-		if (cUserProps && cUserProps[0] && cUserProps[1] && cUserProps[2] && cUserProps[3]
-		&& atoi (cUserProps[2]) >= 1000  // heuristic: first user has an udi of 1000, and other users an uid >= 1000
-		&& strcmp (cUserProps[0], "nobody") != 0)  // remove the 'nobody' user (uid=65534)
+		if (cUserProps && cUserProps[0] && cUserProps[1] && cUserProps[2]
+			&& atoi (cUserProps[2]) >= 1000 && atoi (cUserProps[2]) < 65530)  // heuristic: first user has an udi of 1000, and other users an uid >= 1000; remove the 'nobody' user (uid=65534)
 		{
 			pUser = g_new0 (CDUser, 1);
 			pUser->cUserName = g_strdup (cUserProps[0]);
@@ -883,6 +992,86 @@ static GList* _get_users_list_fallback (void)
 	g_strfreev (cUsers);
 	return pUserList;
 }
+
+static GList* _get_users_list_gdm (void)
+{
+	GError *error = NULL;
+	DBusGProxy *pProxy = cairo_dock_create_new_system_proxy ("org.gnome.DisplayManager",
+		"/org/gnome/DisplayManager/UserManager",
+		"org.gnome.DisplayManager.UserManager");
+	GArray *users =  NULL;
+	dbus_g_proxy_call (pProxy, "GetUserList", &error,
+		G_TYPE_INVALID,
+		dbus_g_type_get_collection ("GArray", G_TYPE_INT64), &users,
+		G_TYPE_INVALID);
+	if (error)
+	{
+		cd_warning ("Couldn't get users on the bus from org.gnome.DisplayManager (%s)\n-> Using a fallback method.", error->message);
+		g_error_free (error);
+		users =  NULL;
+	}
+	if (users == NULL)
+		return _get_users_list_fallback ();
+	
+	GType g_type_ptrarray = dbus_g_type_get_collection ("GPtrArray",
+		dbus_g_type_get_struct("GValueArray",
+			G_TYPE_INT64,  // uid
+			G_TYPE_STRING,  // user name
+			G_TYPE_STRING,  // real name
+			G_TYPE_STRING,  // shell
+			G_TYPE_INT,  // login frequency
+			G_TYPE_STRING,  // icon URL
+			G_TYPE_INVALID));
+	GPtrArray *info =  NULL;
+	dbus_g_proxy_call (pProxy, "GetUsersInfo", &error,
+		dbus_g_type_get_collection ("GArray", G_TYPE_INT64), users,
+		G_TYPE_INVALID,
+		g_type_ptrarray, &info,
+		G_TYPE_INVALID);
+	if (error)
+	{
+		cd_warning ("Couldn't get info on the bus from org.gnome.DisplayManager (%s)\n-> Using a fallback method.", error->message);
+		g_error_free (error);
+		info =  NULL;
+	}
+	if (info == NULL)
+		return _get_users_list_fallback ();
+	
+	CDUser *pUser;
+	GList *pUserList = NULL;
+	GValueArray *va;
+	GValue *v;
+	guint i;
+	for (i = 0; i < info->len; i ++)
+	{
+		va = info->pdata[i];
+		if (! va)
+			continue;
+		
+		pUser = g_new0 (CDUser, 1);
+		v = g_value_array_get_nth (va, 1);  // GValueArray is deprecated from 2.32, yet it's so convenient to map the g_type_ptrarray type ...
+		if (v && G_VALUE_HOLDS_STRING (v))
+			pUser->cUserName = g_strdup (g_value_get_string (v));
+		if (pUser->cUserName == NULL) // shouldn't happen
+			continue;
+		
+		v = g_value_array_get_nth (va, 2);
+		if (v && G_VALUE_HOLDS_STRING (v))
+			pUser->cRealName = g_strdup (g_value_get_string (v));
+		
+		v = g_value_array_get_nth (va, 5);
+		if (v && G_VALUE_HOLDS_STRING (v))
+			pUser->cIconFile = g_strdup (g_value_get_string (v));
+		
+		pUserList = g_list_insert_sorted (pUserList, pUser, (GCompareFunc)_compare_user_name);
+	}
+	g_ptr_array_free (info, TRUE);
+	g_array_free (users, TRUE);
+	
+	g_object_unref (pProxy);
+	return pUserList;
+}
+
 GList *cd_logout_get_users_list (void)
 {
 	// get the list of users
@@ -900,12 +1089,12 @@ GList *cd_logout_get_users_list (void)
 	
 	if (error)
 	{
-		cd_warning ("Couldn't get info on the bus from org.freedesktop.Accounts (%s)\n-> Using a fallback method.", error->message);
+		cd_warning ("Couldn't get info on the bus from org.freedesktop.Accounts (%s)\n-> Trying from GnomeDisplayManager.", error->message);
 		g_error_free (error);
-		return _get_users_list_fallback ();
+		users =  NULL;
 	}
 	if (users == NULL)
-		return _get_users_list_fallback ();
+		return _get_users_list_gdm ();
 	
 	// foreach user, get its properties (name & icon).
 	CDUser *pUser;
