@@ -143,7 +143,7 @@ void cd_shortcuts_on_bookmarks_event (CairoDockFMEventType iEventType, const gch
 							g_free (cGuessedName);
 						}
 						if (cRealURI == NULL)
-							cRealURI = g_strdup ("none");
+							cRealURI = g_strdup (cOneBookmark);
 						if (cIconName == NULL)
 							cIconName = cairo_dock_search_icon_s_path (CD_SHORTCUT_DEFAULT_DIRECTORY_ICON_FILENAME, CAIRO_DOCK_DEFAULT_ICON_SIZE);
 						
@@ -184,7 +184,7 @@ void cd_shortcuts_on_bookmarks_event (CairoDockFMEventType iEventType, const gch
 					}
 				}
 			}
-			
+			pIconsList = CD_APPLET_MY_ICONS_LIST;
 			cairo_dock_sort_icons_by_order (pIconsList);  // again, since 'Home Folder' is always the first bookmark, the head of the list won't change even if there are only bookmarks (so we don't need to re-assigne it to the container).
 		}
 		g_free (cBookmarkFilePath);
@@ -342,7 +342,7 @@ void cd_shortcuts_add_one_bookmark (const gchar *cURI)
 	g_free (cBookmarkFilePath);
 }
 
-Icon * _cd_shortcuts_get_icon (gchar *cFileName, const gchar *cUserName, double fCurrentOrder)
+static Icon * _cd_shortcuts_get_icon (gchar *cFileName, const gchar *cUserName, double fCurrentOrder)
 {
 	cd_debug ("New icon: %s, %s, %f", cFileName, cUserName, fCurrentOrder);
 	gchar *cName, *cRealURI, *cIconName;
@@ -355,7 +355,10 @@ Icon * _cd_shortcuts_get_icon (gchar *cFileName, const gchar *cUserName, double
 	if (cUserName != NULL)
 	{
 		g_free (cName);
-		cName = g_strdup (cUserName);
+		if (cName == NULL)  // a bookmark on a unmounted system or a folder that doesn't exist any more
+			cName = g_strdup_printf ("%s\n[%s]", cUserName, D_("Unmounted"));
+		else
+			cName = g_strdup (cUserName);
 	}
 	else if (cName == NULL)  // a bookmark on a unmounted system
 	{
@@ -365,7 +368,7 @@ Icon * _cd_shortcuts_get_icon (gchar *cFileName, const gchar *cUserName, double
 		g_free (cGuessedName);
 	}
 	if (cRealURI == NULL)
-		cRealURI = g_strdup ("none");
+		cRealURI = g_strdup (cFileName);
 	if (cIconName == NULL)
 		cIconName = cairo_dock_search_icon_s_path (
 			CD_SHORTCUT_DEFAULT_DIRECTORY_ICON_FILENAME,
@@ -391,9 +394,12 @@ GList *cd_shortcuts_list_bookmarks (gchar *cBookmarkFilePath, CairoDockModuleIns
 	// Home
 	gchar *cHome = g_strdup_printf ("file://%s", g_getenv ("HOME"));
 	pNewIcon = _cd_shortcuts_get_icon (cHome, D_("Home Folder"), fCurrentOrder++);
-	pNewIcon->iLastCheckTime = 1e9;  // so that this bookmark will never be considered old, and therefore removed.
-	_init_disk_usage (pNewIcon, myApplet);
-	pBookmarkIconList = g_list_append (pBookmarkIconList, pNewIcon);
+	if (pNewIcon != NULL)
+	{
+		pNewIcon->iLastCheckTime = 1e9;  // so that this bookmark will never be considered old, and therefore removed.
+		_init_disk_usage (pNewIcon, myApplet);
+		pBookmarkIconList = g_list_append (pBookmarkIconList, pNewIcon);
+	}
 
 	gchar *cContent = NULL;
 	gsize length = 0;
