@@ -32,12 +32,12 @@ static void _on_volume_mounted (gboolean bMounting, gboolean bSuccess, const gch
 	CairoContainer *pContainer = CD_APPLET_MY_ICONS_LIST_CONTAINER;
 	CD_APPLET_LEAVE_IF_FAIL (pContainer != NULL);
 	
-	Icon *pIcon = cairo_dock_get_icon_with_base_uri (CD_APPLET_MY_ICONS_LIST, cURI);
-	CD_APPLET_LEAVE_IF_FAIL (pIcon != NULL);
-	
 	//g_print ("%s (%s , %d)\n", __func__, cURI, bSuccess);
 	if (! bSuccess)  // en cas de montage reussi, on aura un dialogue via les evenements.
 	{
+		Icon *pIcon = cairo_dock_get_icon_with_base_uri (CD_APPLET_MY_ICONS_LIST, cURI);
+		CD_APPLET_LEAVE_IF_FAIL (pIcon != NULL);
+		
 		cairo_dock_remove_dialog_if_any (pIcon);
 		cairo_dock_show_temporary_dialog_with_icon_printf (
 			bMounting ? _("failed to mount %s") : _("Failed to unmount %s"),
@@ -55,8 +55,12 @@ static void _open_on_mount (gboolean bMounting, gboolean bSuccess, const gchar *
 	CairoContainer *pContainer = CD_APPLET_MY_ICONS_LIST_CONTAINER;
 	CD_APPLET_LEAVE_IF_FAIL (pContainer != NULL);
 	
-	//g_print ("%s (%s)\n", __func__, cURI);
 	Icon *pIcon = cairo_dock_get_icon_with_base_uri (CD_APPLET_MY_ICONS_LIST, cURI);
+	if (pIcon == NULL && g_strcmp0 (myData.cLastDeletedUri, cURI) == 0 && myData.cLastCreatedUri != NULL)  // when mouting a mount point (for instance a file mounted as a loop device), the associated .volume (its actual URI) might be deleted, and a new one is created for the mounted volume (therefore the icon is deleted and a new one is created, with another URI). So we lose the track of the initial icon. That's why we use a trick: we remember the last created volume; it's very likely this one.
+	{
+		cd_debug ("no icon for '%s', trying with '%s'", cURI, myData.cLastCreatedUri);
+		pIcon = cairo_dock_get_icon_with_base_uri (CD_APPLET_MY_ICONS_LIST, myData.cLastCreatedUri);
+	}
 	CD_APPLET_LEAVE_IF_FAIL (pIcon != NULL);
 	
 	if (bSuccess)  // montage reussi.
@@ -82,7 +86,9 @@ CD_APPLET_ON_CLICK_BEGIN
 		if (CD_APPLET_MY_ICONS_LIST == NULL)
 		{
 			cairo_dock_remove_dialog_if_any (myIcon);
-			if (g_iDesktopEnv == CAIRO_DOCK_KDE)
+			if (myData.pTask != NULL) // if it's loading
+				myData.bShowMenuPending = TRUE;
+			else if (g_iDesktopEnv == CAIRO_DOCK_KDE)
 				cairo_dock_show_temporary_dialog_with_icon (D_("Sorry, this applet is not yet available for KDE."), myIcon, myContainer, 6000., "same icon");
 			else
 				cairo_dock_show_temporary_dialog_with_icon (D_("No disks or bookmarks were found."), myIcon, myContainer, 6000., "same icon");
@@ -107,35 +113,17 @@ CD_APPLET_ON_CLICK_BEGIN
 		}
 		else if (CD_APPLET_CLICKED_ICON->iGroup == (CairoDockIconGroup) CD_BOOKMARK_GROUP)  // clic sur un signet, il peut etre place sur un volume non monte.
 		{
+			// check if it's a mounted URI
+			// note: if it's a folder on an unmouned volume, it can't be launched nor mounted; we would need to get its volume first and it's not obvious (Nautilus just hide them until the volume is mounted)
+			gboolean bIsMounted = TRUE;
+			gchar *cTarget = cairo_dock_fm_is_mounted (CD_APPLET_CLICKED_ICON->cCommand, &bIsMounted);
+			cd_debug ("%s is mounted: %d (%s)", CD_APPLET_CLICKED_ICON->cCommand, bIsMounted, cTarget);
+			g_free (cTarget);
 			
-			/// chercher un volume contenant le signet
-			
-			/// si trouve et non monte, on le monte
-			
-			/// si non trouve, on cree l'URI et on monte.
-			
-			/*gchar *cVolumeUri = g_strdup (CD_APPLET_CLICKED_ICON->cCommand);
-			gchar *str = strchr (cVolumeUri, ':');
-			if (str && strlen (str) > 3)
-			{
-				str = strchr (str, '/');
-			}
-			//g_print ("bookmark '%s' on '%s'\n", CD_APPLET_CLICKED_ICON->cCommand, cVolumeUri);
-			gboolean bIsMounted = FALSE;
-			gchar *cActivationURI = cairo_dock_fm_is_mounted (CD_APPLET_CLICKED_ICON->cBaseURI, &bIsMounted);
-			g_free (cActivationURI);
-			* 
-			if (!bIsMounted)
-			{
-				//g_print (" volume not mounted\n");
-				cairo_dock_fm_mount_full (cVolumeUri, 1, (CairoDockFMMountCallback) _open_on_mount, myApplet);
-			}
-			else
-			{
+			if (bIsMounted)  // if mounted, just open it
 				cairo_dock_fm_launch_uri (CD_APPLET_CLICKED_ICON->cCommand);
-			}
-			g_free (cVolumeUri);*/
-			cairo_dock_fm_launch_uri (CD_APPLET_CLICKED_ICON->cCommand);
+			else  // else, mount it, and it will be opened in the callback.
+				cairo_dock_fm_mount_full (CD_APPLET_CLICKED_ICON->cCommand, 1, (CairoDockFMMountCallback) _open_on_mount, myApplet);
 		}
 		else
 		{
@@ -240,6 +228,7 @@ static void _cd_shortcuts_show_disk_info (GtkMenuItem *menu_item, gpointer *data
 	Icon *pIcon = data[1];
 	CairoContainer *pContainer = data[2];
 	
+	//g_print ("pIcon->cCommand:%s\n", pIcon->cCommand);
 	gchar *cInfo = cd_shortcuts_get_disk_info (pIcon->cCommand, pIcon->cName);
 	cairo_dock_show_temporary_dialog_with_icon (cInfo, pIcon, pContainer, 15000, "same icon");
 	g_free (cInfo);
