@@ -50,24 +50,23 @@ static void _cd_switcher_draw_windows_on_viewport (Icon *pIcon, CDSwitcherDeskto
 	
 	// On calcule les coordonnees en repere absolu.
 	int x = pIcon->windowGeometry.x;  // par rapport au viewport courant.
-	x += myData.switcher.iCurrentViewportX * g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL];  // repere absolu
+	x += myData.switcher.iCurrentViewportX * g_desktopGeometry.Xscreen.width;  // repere absolu
 	if (x < 0)
-		x += g_desktopGeometry.iNbViewportX * g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL];
+		x += g_desktopGeometry.iNbViewportX * g_desktopGeometry.Xscreen.width;
 	int y = pIcon->windowGeometry.y;
-	y += myData.switcher.iCurrentViewportY * g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL];
+	y += myData.switcher.iCurrentViewportY * g_desktopGeometry.Xscreen.height;
 	if (y < 0)
-		y += g_desktopGeometry.iNbViewportY * g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL];
+		y += g_desktopGeometry.iNbViewportY * g_desktopGeometry.Xscreen.height;
 	int w = pIcon->windowGeometry.width, h = pIcon->windowGeometry.height;
 	
 	// test d'intersection avec le viewport donne.
 	//g_print (" %s : (%d;%d) %dx%d\n", pIcon->cName, x, y, w, h);
 	if ((pIcon->iNumDesktop != -1 && pIcon->iNumDesktop != iNumDesktop) ||
-		x + w <= iNumViewportX * g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] ||
-		x >= (iNumViewportX + 1) * g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] ||
-		y + h <= iNumViewportY * g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] ||
-		y >= (iNumViewportY + 1) * g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL])
+		x + w <= iNumViewportX * g_desktopGeometry.Xscreen.width ||
+		x >= (iNumViewportX + 1) * g_desktopGeometry.Xscreen.width ||
+		y + h <= iNumViewportY * g_desktopGeometry.Xscreen.height ||
+		y >= (iNumViewportY + 1) * g_desktopGeometry.Xscreen.height)
 		return ;
-	//g_print (" > on la dessine (%x)\n", pIcon->pIconBuffer);
 	
 	// on dessine ses traits.
 	cairo_save (pCairoContext);
@@ -79,10 +78,10 @@ static void _cd_switcher_draw_windows_on_viewport (Icon *pIcon, CDSwitcherDeskto
 	else
 		cairo_set_source_rgba (pCairoContext, myConfig.RGBWLineColors[0], myConfig.RGBWLineColors[1], myConfig.RGBWLineColors[2], myConfig.RGBWLineColors[3]);
 	cairo_rectangle (pCairoContext,
-		(1.*x/g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] - iNumViewportX)*iOneViewportWidth,
-		(1.*y/g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - iNumViewportY)*iOneViewportHeight,
-		1.*w/g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL]*iOneViewportWidth,
-		1.*h/g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL]*iOneViewportHeight);
+		(1.*x/g_desktopGeometry.Xscreen.width - iNumViewportX)*iOneViewportWidth,
+		(1.*y/g_desktopGeometry.Xscreen.height - iNumViewportY)*iOneViewportHeight,
+		1.*w/g_desktopGeometry.Xscreen.width*iOneViewportWidth,
+		1.*h/g_desktopGeometry.Xscreen.height*iOneViewportHeight);
 
 	if (myConfig.bFillAllWindows || pIcon->Xid == iActiveWindow)
 	{
@@ -94,36 +93,22 @@ static void _cd_switcher_draw_windows_on_viewport (Icon *pIcon, CDSwitcherDeskto
 		cairo_stroke (pCairoContext);
 	}
 	
-	/**Icon *pDisplayedIcon = pIcon;
-	if (pDisplayedIcon->pIconBuffer == NULL)  // icon not loaded (because not in a dock, because inhibited)
-	{
-		
-		/// TODO: we'll have to load the icons that are not loaded (not inside a dock)...
-		Icon *pInhibitor = cairo_dock_get_inhibitor (pIcon, TRUE);  // TRUE = only in dock.
-		if (pInhibitor != NULL)
-			pDisplayedIcon = pInhibitor;
-	}
-	if (pDisplayedIcon->pIconBuffer != NULL)*/
-
 	if (myConfig.bDrawIcons)
 	{
 		const CairoDockImageBuffer *pImage = cairo_dock_appli_get_image_buffer (pIcon);
 		if (pImage && pImage->pSurface)
 		{
-			///int iWidth, iHeight;
-			///cairo_dock_get_icon_extent (pDisplayedIcon, &iWidth, &iHeight);
-			double fZoomX = (double) w/g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL]*iOneViewportWidth / pImage->iWidth;
-			double fZoomY = (double) h/g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL]*iOneViewportHeight / pImage->iHeight;
+			double fZoomX = (double) w/g_desktopGeometry.Xscreen.width*iOneViewportWidth / pImage->iWidth;
+			double fZoomY = (double) h/g_desktopGeometry.Xscreen.height*iOneViewportHeight / pImage->iHeight;
 			double fZoom = MIN (fZoomX, fZoomY);  // on garde le ratio.
 			
 			cairo_translate (pCairoContext,
-				(1.*x/g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] - iNumViewportX)*iOneViewportWidth + (fZoomX - fZoom) * pImage->iWidth/2,
-				(1.*y/g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - iNumViewportY)*iOneViewportHeight + (fZoomY - fZoom) * pImage->iHeight/2);
+				(1.*x/g_desktopGeometry.Xscreen.width - iNumViewportX)*iOneViewportWidth + (fZoomX - fZoom) * pImage->iWidth/2,
+				(1.*y/g_desktopGeometry.Xscreen.height - iNumViewportY)*iOneViewportHeight + (fZoomY - fZoom) * pImage->iHeight/2);
 			cairo_scale (pCairoContext,
 				fZoom,
 				fZoom);
 			cairo_set_source_surface (pCairoContext,
-				///pDisplayedIcon->pIconBuffer,
 				pImage->pSurface,
 				0.,
 				0.);
@@ -150,10 +135,11 @@ void cd_switcher_draw_main_icon_compact_mode (void)
 {
 	if (myData.switcher.iNbColumns == 0 || myData.switcher.iNbLines == 0)  // may happen in desklet mode with a cube desktop, when the desklet is still 0x0.
 		return;
-	g_return_if_fail (myDrawContext != NULL);
+	CD_APPLET_START_DRAWING_MY_ICON_OR_RETURN_CAIRO ();
+	///g_return_if_fail (myDrawContext != NULL);
 	//g_print ("%s (%d;%d)\n", __func__, myData.switcher.iCurrentLine, myData.switcher.iCurrentColumn);
 	// On efface l'icone.
-	cairo_dock_erase_cairo_context (myDrawContext);
+	///cairo_dock_erase_cairo_context (myDrawContext);
 	
 	// definition des parametres de dessin.
 	int iWidth, iHeight;
@@ -165,7 +151,7 @@ void cd_switcher_draw_main_icon_compact_mode (void)
 	double w = iWidth, h = iHeight;
 	if (myConfig.bPreserveScreenRatio)
 	{
-		double r = (double) g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] / g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL];
+		double r = (double) g_desktopGeometry.Xscreen.width / g_desktopGeometry.Xscreen.height;
 		double r_ = myData.switcher.fOneViewportWidth / myData.switcher.fOneViewportHeight;
 		if (r_ > r)  // on etire trop en largeur.
 		{
@@ -341,10 +327,9 @@ void cd_switcher_draw_main_icon_compact_mode (void)
 	cairo_restore (myDrawContext);
 	g_list_free (pWindowList);  // le contenu appartient a la hash table, mais pas la liste.
 	
-	if (CD_APPLET_MY_CONTAINER_IS_OPENGL)
-		cairo_dock_update_icon_texture (myIcon);
-	/**else
-		CD_APPLET_UPDATE_REFLECT_ON_MY_ICON;*/
+	CD_APPLET_FINISH_DRAWING_MY_ICON_CAIRO;
+	///if (CD_APPLET_MY_CONTAINER_IS_OPENGL)
+	///	cairo_dock_update_icon_texture (myIcon);
 }
 
 
@@ -410,7 +395,7 @@ void cd_switcher_draw_main_icon_expanded_mode (void)
 			pIcon = ic->data;
 			cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
 			
-			pCairoContext = cairo_create (pIcon->pIconBuffer);
+			pCairoContext = cairo_create (pIcon->image.pSurface);
 			cairo_set_line_width (pCairoContext, 1.);
 			cairo_set_source_rgba (pCairoContext, myConfig.RGBWLineColors[0], myConfig.RGBWLineColors[1], myConfig.RGBWLineColors[2], myConfig.RGBWLineColors[3]);
 			
@@ -527,7 +512,7 @@ static void _show_desktop (GtkMenuItem *menu_item, gpointer data)
 	CD_APPLET_ENTER;
 	int iIndex = GPOINTER_TO_INT (data);
 	int iNumDesktop, iNumViewportX, iNumViewportY;
-	cd_switcher_compute_viewports_from_index (iIndex, &iNumDesktop, &iNumViewportX, &iNumViewportY);
+	cd_switcher_compute_desktop_from_index (iIndex, &iNumDesktop, &iNumViewportX, &iNumViewportY);
 	if (iNumDesktop != myData.switcher.iCurrentDesktop)
 		cairo_dock_set_current_desktop (iNumDesktop);
 	if (iNumViewportX != myData.switcher.iCurrentViewportX || iNumViewportY != myData.switcher.iCurrentViewportY)
@@ -537,19 +522,10 @@ static void _show_desktop (GtkMenuItem *menu_item, gpointer data)
 static void _cd_switcher_list_window_on_viewport (Icon *pIcon, int iNumDesktop, int iNumViewportX, int iNumViewportY, GtkWidget *pMenu)
 {
 	//g_print (" + %s\n", pIcon->cName);
-	// on recupere la taille de l'icone.
-	/**Icon *pDisplayedIcon = pIcon;
-	if (pDisplayedIcon->pIconBuffer == NULL)  // icon not loaded (because not in a dock, because inhibited)
-	{
-		Icon *pInhibitor = cairo_dock_get_inhibitor (pIcon, TRUE);  // TRUE = only in dock.
-		if (pInhibitor != NULL)
-			pDisplayedIcon = pInhibitor;
-	}
-	const CairoDockImageBuffer *pImage = cairo_dock_appli_get_image_buffer (pIcon);*/
 	
 	// on cree une copie de la surface de l'icone a la taille du menu.
-	GdkPixbuf *pixbuf = cairo_dock_icon_buffer_to_pixbuf (pIcon/**pDisplayedIcon*/);
-	if (pixbuf == NULL)
+	GdkPixbuf *pixbuf = cairo_dock_icon_buffer_to_pixbuf (pIcon);
+	if (pixbuf == NULL)  // icon not loaded (because not in a dock, because inhibited)
 	{
 		const gchar *cIcon = cairo_dock_get_class_icon (pIcon->cClass);
 		gint iDesiredIconSize = cairo_dock_search_icon_size (GTK_ICON_SIZE_LARGE_TOOLBAR); // 24px
@@ -585,7 +561,7 @@ void cd_switcher_build_windows_list (GtkWidget *pMenu)
 	// chaque bureau/viewport.
 	int iNumDesktop=0, iNumViewportX=0, iNumViewportY=0;
 	int k = 0, N = g_desktopGeometry.iNbDesktops * g_desktopGeometry.iNbViewportX * g_desktopGeometry.iNbViewportY;
-	int iIndex = cd_switcher_compute_index (myData.switcher.iCurrentDesktop, myData.switcher.iCurrentViewportX, myData.switcher.iCurrentViewportY);
+	int iIndex = cd_switcher_compute_index_from_desktop (myData.switcher.iCurrentDesktop, myData.switcher.iCurrentViewportX, myData.switcher.iCurrentViewportY);
 	GString *sDesktopName = g_string_new ("");
 	int i, j;
 	for (j = 0; j < myData.switcher.iNbLines; j ++)
@@ -597,12 +573,12 @@ void cd_switcher_build_windows_list (GtkWidget *pMenu)
 			gtk_menu_shell_append(GTK_MENU_SHELL (pMenu), pMenuItem);
 			g_object_set (pMenuItem, "height-request", 3, NULL);
 			
-			if (k < myConfig.iNbNames)
+			if (k < myData.iNbNames)
 			{
 				if (k == iIndex)
-					g_string_printf (sDesktopName, "<b>%s (%s)</b>", myConfig.cDesktopNames[k], D_("Current"));
+					g_string_printf (sDesktopName, "<b>%s (%s)</b>", myData.cDesktopNames[k], D_("Current"));
 				else
-					g_string_printf (sDesktopName, "<b>%s</b>", myConfig.cDesktopNames[k]);
+					g_string_printf (sDesktopName, "<b>%s</b>", myData.cDesktopNames[k]);
 			}
 			else
 			{
@@ -661,8 +637,8 @@ static void _cd_switcher_move_window_to_viewport (Icon *pIcon, int iNumDesktop,
 	
 	cairo_dock_move_xwindow_to_nth_desktop (pIcon->Xid,
 		iDestNumDesktop,
-		(iDestNumViewportX - myData.switcher.iCurrentViewportX) * g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL],
-		(iDestNumViewportY - myData.switcher.iCurrentViewportY) * g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL]);
+		(iDestNumViewportX - myData.switcher.iCurrentViewportX) * g_desktopGeometry.Xscreen.width,
+		(iDestNumViewportY - myData.switcher.iCurrentViewportY) * g_desktopGeometry.Xscreen.height);
 }
 void cd_switcher_move_current_desktop_to (int iNumDesktop, int iNumViewportX, int iNumViewportY)
 {
