@@ -136,13 +136,12 @@ static gboolean _cd_switcher_get_viewport_from_clic (Icon *pClickedIcon, int *iN
 		int iNumLine = (int) (iMouseY / (h) * myData.switcher.iNbLines);
 		int iNumColumn = (int) (iMouseX / (w) * myData.switcher.iNbColumns);
 		cd_switcher_compute_desktop_from_coordinates (iNumLine, iNumColumn, iNumDesktop, iNumViewportX, iNumViewportY);
-		//g_print ("(%d;%d) --> (%d;%d;%d)\n", iNumLine, iNumColumn, *iNumDesktop, *iNumViewportX, *iNumViewportY);
 		return TRUE;
 	}
 	else if (pClickedIcon != NULL && pClickedIcon != myIcon)
 	{
 		int iIndex = pClickedIcon->fOrder;
-		cd_switcher_compute_viewports_from_index (iIndex, iNumDesktop, iNumViewportX, iNumViewportY);
+		cd_switcher_compute_desktop_from_index (iIndex, iNumDesktop, iNumViewportX, iNumViewportY);
 		return TRUE;
 	}
 	else
@@ -162,7 +161,7 @@ CD_APPLET_ON_CLICK_END
 
 
 CD_APPLET_ON_SCROLL_BEGIN  // Merci ChangFu !
-	int iIndex = cd_switcher_compute_index (myData.switcher.iCurrentDesktop, myData.switcher.iCurrentViewportX, myData.switcher.iCurrentViewportY);
+	int iIndex = cd_switcher_compute_index_from_desktop (myData.switcher.iCurrentDesktop, myData.switcher.iCurrentViewportX, myData.switcher.iCurrentViewportY);
 	int iNumDesktop, iNumViewportX, iNumViewportY;
 	cd_debug ("Switcher: current %d", iIndex);
 	if (CD_APPLET_SCROLL_DOWN)
@@ -170,14 +169,14 @@ CD_APPLET_ON_SCROLL_BEGIN  // Merci ChangFu !
 		iIndex++;
 		if (iIndex >= myData.switcher.iNbViewportTotal)
 			iIndex = 0;
-		cd_switcher_compute_viewports_from_index (iIndex, &iNumDesktop, &iNumViewportX, &iNumViewportY);
+		cd_switcher_compute_desktop_from_index (iIndex, &iNumDesktop, &iNumViewportX, &iNumViewportY);
 	}
 	else if (CD_APPLET_SCROLL_UP)
 	{
 		iIndex = iIndex - 1;
 		if (iIndex < 0)
 			iIndex = myData.switcher.iNbViewportTotal - 1;
-		cd_switcher_compute_viewports_from_index (iIndex, &iNumDesktop, &iNumViewportX, &iNumViewportY);
+		cd_switcher_compute_desktop_from_index (iIndex, &iNumDesktop, &iNumViewportX, &iNumViewportY);
 	}
 	else
 		CD_APPLET_LEAVE (CAIRO_DOCK_LET_PASS_NOTIFICATION);
@@ -206,7 +205,7 @@ static void _cd_switcher_move_to_desktop (GtkMenuItem *menu_item, gpointer data)
 {
 	int iIndex = GPOINTER_TO_INT (data);
 	int iNumDesktop, iNumViewportX, iNumViewportY;
-	cd_switcher_compute_viewports_from_index (iIndex, &iNumDesktop, &iNumViewportX, &iNumViewportY);
+	cd_switcher_compute_desktop_from_index (iIndex, &iNumDesktop, &iNumViewportX, &iNumViewportY);
 	cd_switcher_move_current_desktop_to (iNumDesktop, iNumViewportX, iNumViewportY);
 }
 
@@ -220,41 +219,32 @@ static void _on_got_workspace_name (int iClickedButton, GtkWidget *pInteractiveW
 		if (cNewName != NULL)
 		{
 			int iIndex = GPOINTER_TO_INT (data);
-			// on augmente la taille du tableau si necessaire.
-			if (iIndex >= myConfig.iNbNames)
+			// store the new name inside the array.
+			if (iIndex >= myData.iNbNames)
 			{
-				myConfig.cDesktopNames = g_realloc (myConfig.cDesktopNames, (iIndex + 2) * sizeof (gchar*));  // NULL-terminated.
+				myData.cDesktopNames = g_realloc (myData.cDesktopNames, (iIndex + 2) * sizeof (gchar*));  // NULL-terminated.
 				int i;
-				for (i = myConfig.iNbNames; i < iIndex; i ++)  // on met des noms par defaut aux nouveaux, sauf a celui choisi.
-					myConfig.cDesktopNames[i] = g_strdup_printf ("%s %d", D_("Desktop"), i+1);
-				myConfig.cDesktopNames[iIndex] = NULL;
-				myConfig.cDesktopNames[iIndex+1] = NULL;  // NULL-terminated.
-				myConfig.iNbNames = iIndex + 1;
+				for (i = myData.iNbNames; i < iIndex; i ++)  // on met des noms par defaut aux nouveaux, sauf a celui choisi.
+					myData.cDesktopNames[i] = g_strdup_printf ("%s %d", D_("Desktop"), i+1);
+				myData.cDesktopNames[iIndex] = NULL;
+				myData.cDesktopNames[iIndex+1] = NULL;  // NULL-terminated.
+				myData.iNbNames = iIndex + 1;
 			}
 			
-			g_free (myConfig.cDesktopNames[iIndex]);
-			myConfig.cDesktopNames[iIndex] = g_strdup (cNewName);  // donc ne pas liberer 'cNewName'.
+			g_free (myData.cDesktopNames[iIndex]);
+			myData.cDesktopNames[iIndex] = g_strdup (cNewName);  // donc ne pas liberer 'cNewName'.
 			
-			// on sauvegarde le nom dans le fichier de conf.
-			GString *sNames = g_string_new ("");
-			int i;
-			for (i = 0; i < myConfig.iNbNames; i ++)
-			{
-				g_string_append_printf (sNames, "%s;", myConfig.cDesktopNames[i]);
-			}
-			sNames->str[sNames->len-1] = '\0';
-			cairo_dock_update_conf_file (CD_APPLET_MY_CONF_FILE,
-				G_TYPE_STRING, "Configuration", "desktop names", sNames->str,
-				G_TYPE_INVALID);
-			g_string_free (sNames, TRUE);
+			// apply the change on X (it will trigger the notification, which will store the names in config).
+			cairo_dock_set_desktops_names (myData.cDesktopNames);
 		}
 	}
 	CD_APPLET_LEAVE ();
-}static void _cd_switcher_rename_desktop (GtkMenuItem *menu_item, gpointer data)
+}
+static void _cd_switcher_rename_desktop (GtkMenuItem *menu_item, gpointer data)
 {
 	// on demande le nouveau nom.
 	int iIndex = GPOINTER_TO_INT (data);
-	gchar *cName = (iIndex < myConfig.iNbNames ? g_strdup (myConfig.cDesktopNames[iIndex]) : g_strdup_printf ("%s %d", D_("Desktop"), iIndex+1));
+	gchar *cName = (iIndex < myData.iNbNames ? g_strdup (myData.cDesktopNames[iIndex]) : g_strdup_printf ("%s %d", D_("Desktop"), iIndex+1));
 	cairo_dock_show_dialog_with_entry (D_("Rename this workspace"),
 		myIcon, myContainer, "same icon",
 		cName,
@@ -288,7 +278,7 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 	int iNumDesktop, iNumViewportX, iNumViewportY;
 	if (_cd_switcher_get_viewport_from_clic (pClickedIcon, &iNumDesktop, &iNumViewportX, &iNumViewportY))
 	{
-		int iIndex = cd_switcher_compute_index (iNumDesktop, iNumViewportX, iNumViewportY);
+		int iIndex = cd_switcher_compute_index_from_desktop (iNumDesktop, iNumViewportX, iNumViewportY);
 		CD_APPLET_ADD_IN_MENU_WITH_STOCK_AND_DATA (D_("Rename this workspace"),
 			GTK_STOCK_EDIT,
 			_cd_switcher_rename_desktop,
@@ -382,10 +372,10 @@ gboolean on_change_desktop (CairoDockModuleInstance *myApplet)
 {
 	CD_APPLET_ENTER;
 	cd_debug ("");
-	int iPreviousIndex = cd_switcher_compute_index (myData.switcher.iCurrentDesktop, myData.switcher.iCurrentViewportX, myData.switcher.iCurrentViewportY);
+	int iPreviousIndex = cd_switcher_compute_index_from_desktop (myData.switcher.iCurrentDesktop, myData.switcher.iCurrentViewportX, myData.switcher.iCurrentViewportY);
 	
 	cd_switcher_get_current_desktop ();
-	int iIndex = cd_switcher_compute_index (myData.switcher.iCurrentDesktop, myData.switcher.iCurrentViewportX, myData.switcher.iCurrentViewportY);
+	int iIndex = cd_switcher_compute_index_from_desktop (myData.switcher.iCurrentDesktop, myData.switcher.iCurrentViewportX, myData.switcher.iCurrentViewportY);
 	
 	if (myConfig.bDisplayNumDesk)
 	{
@@ -414,8 +404,8 @@ gboolean on_change_desktop (CairoDockModuleInstance *myApplet)
 			icon = ic->data;
 			if (icon->fOrder == iPreviousIndex)  // l'ancienne icone du bureau courant.
 			{
-				if (iPreviousIndex < myConfig.iNbNames)
-					cairo_dock_set_icon_name (myConfig.cDesktopNames[iPreviousIndex], icon, pContainer);
+				if (iPreviousIndex < myData.iNbNames)
+					cairo_dock_set_icon_name (myData.cDesktopNames[iPreviousIndex], icon, pContainer);
 				else
 					cairo_dock_set_icon_name_printf (icon, pContainer, "%s %d", D_("Desktop"), iPreviousIndex+1);
 				icon->bHasIndicator = FALSE;
@@ -456,6 +446,34 @@ gboolean on_window_configured (CairoDockModuleInstance *myApplet, Window Xid, XC
 }
 
 
+static void _save_desktop_names (void)
+{
+	GString *sNames = g_string_new ("");
+	int i;
+	for (i = 0; i < myData.iNbNames; i ++)
+	{
+		g_string_append_printf (sNames, "%s;", myData.cDesktopNames[i]);
+	}
+	sNames->str[sNames->len-1] = '\0';
+	cairo_dock_update_conf_file (CD_APPLET_MY_CONF_FILE,
+		G_TYPE_STRING, "Configuration", "desktop names", sNames->str,
+		G_TYPE_INVALID);
+	g_string_free (sNames, TRUE);
+}
+gboolean on_change_desktop_names (CairoDockModuleInstance *myApplet)
+{
+	CD_APPLET_ENTER;
+	// retrieve the desktop names
+	if (myData.cDesktopNames != NULL)
+		g_strfreev (myData.cDesktopNames);
+	myData.cDesktopNames = cairo_dock_get_desktops_names ();
+	myData.iNbNames = g_strv_length (myData.cDesktopNames);
+	// store them in config, to be able to set them on startup if noone has done it.
+	_save_desktop_names ();
+	CD_APPLET_LEAVE (CAIRO_DOCK_LET_PASS_NOTIFICATION);
+}
+
+
 gboolean on_mouse_moved (CairoDockModuleInstance *myApplet, CairoContainer *pContainer, gboolean *bStartAnimation)
 {
 	CD_APPLET_ENTER;
@@ -466,14 +484,14 @@ gboolean on_mouse_moved (CairoDockModuleInstance *myApplet, CairoContainer *pCon
 	if (! _cd_switcher_get_viewport_from_clic (myIcon, &iNumDesktop, &iNumViewportX, &iNumViewportY))
 		CD_APPLET_LEAVE (CAIRO_DOCK_LET_PASS_NOTIFICATION);
 	
-	int iIndex = cd_switcher_compute_index (iNumDesktop, iNumViewportX, iNumViewportY);
+	int iIndex = cd_switcher_compute_index_from_desktop (iNumDesktop, iNumViewportX, iNumViewportY);
 	if (iIndex != myData.iPrevIndexHovered)
 	{
 		myData.iPrevIndexHovered = iIndex;
 		myData.fDesktopNameAlpha = 0.;
-		if (iIndex < myConfig.iNbNames)
+		if (iIndex < myData.iNbNames)
 		{
-			CD_APPLET_SET_NAME_FOR_MY_ICON (myConfig.cDesktopNames[iIndex]);
+			CD_APPLET_SET_NAME_FOR_MY_ICON (myData.cDesktopNames[iIndex]);
 		}
 		else
 		{
