@@ -123,7 +123,7 @@ CD_APPLET_RELOAD_BEGIN
 	if (myDesklet)  // on cloue le desklet.
 	{
 		GdkGravity iGravity;
-		if (myContainer->iWindowPositionX < g_desktopGeometry.iScreenWidth[CAIRO_DOCK_HORIZONTAL]/2)  // on prend la taille de l'ecran ou il est place plutot que la taille totale, car en general on n'utilise pas un dual-screen comme un seul grand ecran maisplutot comme 2 bureaux.
+		if (myContainer->iWindowPositionX < g_desktopGeometry.Xscreen.width/2)  // we don't know if the container is set on a given screen or not, so take the X screen.
 			iGravity = GDK_GRAVITY_NORTH_WEST;
 		else
 			iGravity = GDK_GRAVITY_NORTH_EAST;
